package cz.muni.fi.pa165.rating;

import cz.muni.fi.pa165.movie.Movie;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Optional;

@Service
public class RatingService {

    private final RatingRepository repository;

    private final RatingMapper ratingMapper;

    /**
     * Constructs a new instance of the RatingService class.
     *
     * @param ratingRepository the repository for managing {@link Rating} entities
     * @param ratingMapper the mapper for converting between {@link Rating} and {@link RatingDto}
     */
    @Autowired
    public RatingService(RatingRepository ratingRepository, RatingMapper ratingMapper) {
        this.repository = ratingRepository;
        this.ratingMapper = ratingMapper;
    }

    /**
     * Retrieves all ratings.
     *
     * @return a list of all ratings
     */
    @Transactional(readOnly = true)
    public List<RatingDto> getRatings() {
        return repository.findAll().stream().map(ratingMapper::toDto).toList();
    }

    /**
     * Creates a new rating.
     *
     * @param ratingDto the rating DTO to be created
     */
    @Transactional
    public void createRating(RatingDto ratingDto) {
        Rating rating = ratingMapper.fromDto(ratingDto);
        repository.save(rating);
    }

    /**
     * Updates an existing rating.
     *
     * @param id        the ID of the rating to be updated
     * @param ratingDto the updated rating information
     */
    @Transactional
    public void updateRating(Long id, RatingDto ratingDto) {
        ratingMapper.updateFromDto(ratingDto, repository.getReferenceById(id));
    }

    /**
     * Deletes an existing rating by ID.
     *
     * @param id the ID of the rating to be deleted
     */
    @Transactional
    public void deleteRating(Long id) {
        repository.delete(repository.getReferenceById(id));
    }

    /**
     * Deletes all ratings.
     */
    @Transactional
    public void deleteAllRatings() {
        repository.deleteAll();
    }

    /**
     * Retrieves a rating by its ID.
     *
     * @param id the ID of the rating to retrieve.
     * @return the rating with the specified ID.
     */
    @Transactional(readOnly = true)
    public RatingDto getById(Long id) {
        Optional<Rating> rating = repository.findById(id);
        if (rating.isPresent()) {
            return ratingMapper.toDto(rating.get());
        } else {
            throw new RuntimeException("Rating not found with id: " + id);
        }
    }

    /**
     * Retrieves a ratings for movie.
     *
     * @param movieId the ID of movie which ratings we want.
     * @return  a list of all ratings for movie
     */
    public List<RatingDto> getRatingsForMovie(long movieId) {
        return repository.findAll().stream().filter(rating -> rating.getMovieId() == movieId).map(ratingMapper::toDto).toList();
    }
}
