package cz.muni.fi.pa165;

import cz.muni.fi.pa165.movie.MovieDto;
import cz.muni.fi.pa165.rating.RatingDto;
import org.assertj.core.util.VisibleForTesting;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

/**
 * This code was written by Jiří Smejkal, I just rewrote the code to make it more readable
 **/
@Service
public class RecommenderService {

    @Value("${CORE_APP_URL:http://localhost:8090}")
    private String coreAppUrl;

    private final RestTemplate restTemplate;

    /**
     * Constructs a new instance of the {@code RecommenderService} class with the specified REST template.
     *
     * @param restTemplate the REST template used to perform HTTP requests
     */
    @Autowired
    public RecommenderService(RestTemplate restTemplate) {
        this.restTemplate = restTemplate;
    }

    /**
     * Retrieves a list of movies that are similar to the specified movie, based on their genres and ratings.
     * Returns the top 5 movies with the highest similarity scores.
     *
     * @param movieId     the ID of the movie to find similar movies for
     * @param accessToken the access token used to authenticate the request
     * @return a list of movies that are similar to the specified movie
     */
    public List<MovieDto> getSimilarMovies(Long movieId, String accessToken) {
        String moviesUrl = coreAppUrl + "/movies";
        String ratingsUrl = coreAppUrl + "/ratings";

        HttpEntity<?> entity = null;

        if (accessToken != null) {
            HttpHeaders headers = new HttpHeaders();
            headers.set("Authorization", "Bearer " + accessToken);
            entity = new HttpEntity<>(headers);
        }

        ResponseEntity<List<MovieDto>> moviesResponse = restTemplate.exchange(
                moviesUrl,
                HttpMethod.GET,
                entity,
                new ParameterizedTypeReference<>() {
                });
        List<MovieDto> allMovies = moviesResponse.getBody();
        if (allMovies == null || allMovies.isEmpty()) {
            return Collections.emptyList();
        }

        ResponseEntity<List<RatingDto>> ratingsResponse = restTemplate.exchange(
                ratingsUrl,
                HttpMethod.GET,
                entity,
                new ParameterizedTypeReference<>() {
                });
        List<RatingDto> allRatings = ratingsResponse.getBody();
        if (allRatings == null || allRatings.isEmpty()) {
            return Collections.emptyList();
        }

        MovieDto targetMovie = allMovies.stream()
                .filter(movie -> movie.getId() == movieId)
                .findFirst()
                .orElse(null);

        if (targetMovie == null) {
            return Collections.emptyList();
        }

        List<RatingDto> targetRatings = allRatings.stream()
                .filter(rating -> rating.getMovieId() == movieId)
                .collect(Collectors.toList());

        List<MovieSimilarity> similarities = allMovies.stream()
                .filter(movie -> movie.getId() != movieId)
                .map(movie -> {
                    double ratingSimilarity = calculateRatingSimilarity(targetRatings, allRatings, movie.getId());
                    double genreSimilarity = calculateGenreSimilarity(targetMovie, movie);
                    double overallSimilarity = ratingSimilarity * 0.6 + genreSimilarity * 0.4;

                    return new MovieSimilarity(movie, overallSimilarity);
                }).sorted().collect(Collectors.toList());

        return similarities.stream()
                .limit(5)
                .map(similarity -> similarity.movie)
                .collect(Collectors.toList());
    }


    private record MovieSimilarity(MovieDto movie, double similarityScore) implements Comparable<MovieSimilarity> {
        @Override
        public int compareTo(MovieSimilarity other) {
            return Double.compare(other.similarityScore, this.similarityScore);
        }
    }


    private double calculateRatingSimilarity(List<RatingDto> targetRatings, List<RatingDto> allRatings, Long otherMovieId) {
        List<RatingDto> otherRatings = allRatings.stream()
                .filter(rating -> rating.getMovieId() == otherMovieId)
                .collect(Collectors.toList());

        double sumSquaredDifferences = targetRatings.stream()
                .flatMap(targetRating -> otherRatings.stream()
                        .filter(otherRating -> targetRating.getUserId() == otherRating.getUserId() && targetRating.getCategory() == otherRating.getCategory())
                        .map(otherRating -> targetRating.getRating() - otherRating.getRating())
                        .findFirst().stream())
                .mapToDouble(difference -> difference * difference)
                .sum();

        if (targetRatings.isEmpty()) {
            return 0;
        }
        return 1.0 / (1.0 + Math.sqrt(sumSquaredDifferences / targetRatings.size()));

    }


    private double calculateGenreSimilarity(MovieDto targetMovie, MovieDto otherMovie) {
        double intersection = targetMovie.getGenres().stream()
                .filter(otherMovie.getGenres()::contains)
                .count();
        double union = targetMovie.getGenres().size() + otherMovie.getGenres().size();
        if (union == 0) {
            return 0;
        }
        union -= intersection;
        return intersection / union;
    }

    /**
     * Calculates the similarity between the ratings of a target movie and the ratings of another movie.
     * This method is intended for testing purposes only.
     *
     * @param targetRatings the ratings of the target movie
     * @param allRatings    all ratings in the system
     * @param otherMovieId  the ID of the movie to compare ratings with
     * @return a double representing the similarity score between 0.0 and 1.0, where 1.0 means the movies have the same ratings
     */
    @VisibleForTesting
    public double getRatingsSimilarityForTesting(List<RatingDto> targetRatings, List<RatingDto> allRatings, Long otherMovieId) {
        return calculateRatingSimilarity(targetRatings, allRatings, otherMovieId);
    }

    /**
     * Calculates the similarity between the genres of two movies based on the number of genres they share.
     * This method is intended for testing purposes only.
     *
     * @param targetMovie the first movie to compare genres with
     * @param otherMovie  the second movie to compare genres with
     * @return a double representing the similarity score between 0.0 and 1.0, where 1.0 means the movies have the same genres
     */
    @VisibleForTesting
    public double getGenresSimilarityForTesting(MovieDto targetMovie, MovieDto otherMovie) {
        return calculateGenreSimilarity(targetMovie, otherMovie);
    }
}
